<?php

    /**
     * Logging class:
     * - contains lfile, lwrite and lclose public methods
     * - lfile sets path and name of log file
     * - lwrite writes message to the log file (and implicitly opens log file)
     * - lclose closes log file
     * - first call of lwrite method will open log file implicitly
     * - message is written with the following format: [d/M/Y:H:i:s] (script name) message
     */
    if (!class_exists("In8_Logger")) {

        class In8_Logger {

// declare log file and file pointer as private properties
            private $fp;
            public $disableLogger = FALSE;
            public $log_file, $log_file_url; //log file
            //array of default emails which would recieve all error emails
            private $core_emails = ['support@in8sync.com'];

// set log file
            public function lfile($path, $url) {
                $this->log_file_url = $url;
                $this->log_file = $path;
            }

// write message to the log file
            public function lwrite($message, $is_error = false) {
                if ($this->disableLogger === TRUE) {
                    return FALSE;
                }
// if file pointer doesn't exist, then open log file
                if (!is_resource($this->fp)) {
                    $this->lopen();
                }

// define script name
                $script_name = pathinfo($_SERVER['PHP_SELF'], PATHINFO_FILENAME);
// define current time and suppress E_WARNING if using the system TZ settings
// (don't forget to set the INI setting date.timezone)
                $time = @date('[d/M/Y:H:i:s]');
// write current time, script name and message to the log file
                fwrite($this->fp, "$time ($script_name) $message" . PHP_EOL);
            }

// close log file (it's always a good idea to close a file when you're done with it)
            public function lclose() {
                fclose($this->fp);
            }

// open log file (private method)
            private function lopen() {
// in case of Windows set default log file
                if (strtoupper(substr(PHP_OS, 0, 3)) === 'WIN') {
                    $log_file_default = 'c:/php/logfile.txt';
                }
// set default log file for Linux and other systems
                else {
                    $log_file_default = '/tmp/logfile.txt';
                }
// define log file from lfile method or use previously set default
                $lfile = $this->log_file ? $this->log_file : $log_file_default;
// open log file for writing only and place file pointer at the end of the file
// (if the file does not exist, try to create it)
                $this->fp = fopen($lfile, 'a');
            }

            /**
             * 
             * @param type $type - 0 for error , 1 for success
             * @param type $record - recordType of the object 
             * @param type $message - log message
             */
            public function processError($error, $error_code, $object, $object_id) {
                $debug_level = IN8_Settings::getDebuggingLevel();
                $email_actions = IN8_Settings::getEmailActions();

                $error = "Level " . $debug_level . " :: " . $error;
                $this->lwrite($error); //write to file

                if (!empty($email_actions)) {
                    foreach ($email_actions as $err_code => $email_action) {
                        if ($error_code == $err_code) {
                            if ($email_action[0] == 0) {
                                return;
                            } else {
                                $emails = $email_action[1];
                            }
                        }
                    }
                    if (isset($emails)) {
                        $this->sendErrorToSpecifiedEmails($emails, $error);
                    } else {
                        $this->sendErrorNotification($object, $error); //send email
                    }
                } else {
                    $this->sendErrorNotification($object, $error); //send email
                }
	
	
	            do_action('in8_error_notification', $error, $object, $object_id, $this->log_file);

                $this->writeLogToDB(0, $error, $object, $object_id); // write to database 
            }

            public function sendErrorToSpecifiedEmails($emails, $message) {
                $emails = explode(",", $emails); //convert to array
                if (!empty($emails)) {
                    $attachments = [];
                    //add log file as attachment to email
                    if ($this->log_file && file_exists($this->log_file)) {
                        $attachments = [$this->log_file];
                    } else {
                        $message .= " ! No log file found associated with this record";
                    }

                    $message_subject = get_site_url() . " : IN8 Error Occured";
                    foreach ($emails as $email) { //loop over emails
                        if (!filter_var($email, FILTER_VALIDATE_EMAIL) === false) { //validate the email
                            wp_mail($email, $message_subject, $message, [], $attachments); //send email
                        }
                    }
                }
            }

            /**
             * write a log to database
             * @global type $wpdb
             * @param type $status
             * @param type $error
             * @return boolean
             */
            public function writeLogToDB($status = 0, $error = '', $object = '', $object_id = '') {
                global $wpdb;
                $query_array = ["status" => $status, "woo_object_id" => $object_id, "operation" => $object];
                $query_array['notes'] = ($status == 0 && !empty($error)) ? esc_sql($error) : 'success';
                $wpdb->insert($wpdb->prefix . "in8sync_log", $query_array);
                return FALSE;
            }

            /*
             * function called by all our scripts
             */

            public function sendErrorNotification($record, $message = '') {
                $message = "Error occured in record " . $record . " :: " . $message;
                $attachments = [];
                //add log file as attachment to email
                if (file_exists($this->log_file)) {
                    $attachments = [$this->log_file];
                } else {
                    $message .= " ! No log file found associated with this record";
                }
                $this->sendErrorEmails($message, $attachments);
            }

            public function sendErrorEmails($message, $attachments = [], $message_subject = "") {
                if (!empty($message)) {
	
					if( !class_exists('in8_tabs') ){
						require WP_PLUGIN_DIR . "/in8sync/code/admin/tabs/tabs.php";
					}

                    //get saved emails
                    $email_settings = in8_settings::getTabSettings(in8_tabs::$tab_prefix."email",'debug_emails');
                    $emails = [];

                    if (!empty($email_settings['default_debug_emails'])) {
                        $emails = explode(",", $email_settings['default_debug_emails']); //convert to array
                    }

                    if (!empty($this->core_emails)) {
                        $emails = array_merge($emails, $this->core_emails);
                    }
                    if (!empty($emails)) {
                        if (empty($message_subject)) {
                            $message_subject = get_site_url() . " : IN8 Error Occured";
                        }
                        foreach ($emails as $email) { //loop over emails
                            if (!filter_var($email, FILTER_VALIDATE_EMAIL) === false) { //validate the email
                                wp_mail($email, $message_subject, $message, [], $attachments); //send email
                            }
                        }
                    }
                }
            }
		
		    public function saveRequestJson($data = array()) {
			    if (!empty($data) && !empty($this->log_file)) {
				    if( !is_array($data) ){
					    $data = @json_decode($data, ARRAY_A);
				    }
				    if( !is_array($data) ){
					    return;
				    }
				
                	$restricted_fields = apply_filters('in8_restrict_fields_from_saved_request', ['creditcards']);
				    foreach($restricted_fields as $field){
					    if( isset($data[$field]) ){
						    unset($data[$field]);
					    }
				    }
				
				    $data_encoded = json_encode($data, JSON_UNESCAPED_SLASHES);
				    $json_file = str_replace(".log", ".json", $this->log_file);
				    $json_file_path = dirname($json_file);
				    @file_put_contents($json_file, $data_encoded);
			    }
		    }
        }

    }
